@@ -86,6 +86,11 @@ typedef enum {
    nir_op_fdot2,
    nir_op_fdot3,
    nir_op_fdot4,
+   nir_op_fdot_replicated2,
+   nir_op_fdot_replicated3,
+   nir_op_fdot_replicated4,
+   nir_op_fdph,
+   nir_op_fdph_replicated,
    nir_op_feq,
    nir_op_fexp2,
    nir_op_ffloor,
@@ -176,9 +181,12 @@ typedef enum {
    nir_op_uge,
    nir_op_ult,
    nir_op_umax,
+   nir_op_umax_4x8,
    nir_op_umin,
+   nir_op_umin_4x8,
    nir_op_umod,
    nir_op_umul_high,
+   nir_op_umul_unorm_4x8,
    nir_op_unpack_half_2x16,
    nir_op_unpack_half_2x16_split_x,
    nir_op_unpack_half_2x16_split_y,
@@ -186,7 +194,9 @@ typedef enum {
    nir_op_unpack_snorm_4x8,
    nir_op_unpack_unorm_2x16,
    nir_op_unpack_unorm_4x8,
+   nir_op_usadd_4x8,
    nir_op_ushr,
+   nir_op_ussub_4x8,
    nir_op_usub_borrow,
    nir_op_vec2,
    nir_op_vec3,
