@@ -175,7 +175,7 @@ union YYSTYPE
    gl_state_index state[STATE_LENGTH];
    int negate;
    struct asm_vector vector;
-   gl_inst_opcode opcode;
+   enum prog_opcode opcode;
 
    struct {
       unsigned swz;
@@ -184,7 +184,7 @@ union YYSTYPE
       unsigned negate:1;
    } ext_swizzle;
 
-#line 188 "program/program_parse.tab.h" /* yacc.c:1909  */
+#line 188 "./program/program_parse.tab.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
